% Lens design

% Christian Walther, diploma thesis 2005

function compute()

try
    handles = guidata(evalin('base', 'cwldfigure'));
catch
    errordlg('Please run ''design'' before running ''compute''.');
    return
end

if (length(handles.beams) == 0)
    errordlg('No beams defined.');
    return
end

wp = zeros(length(handles.lenses) + 1, length(handles.beams));
lp = zeros(1, length(handles.lenses));
dw = zeros(length(handles.lenses) + 1, length(handles.beams));
dl = zeros(length(handles.lenses), length(handles.beams));

leff = [];
z = [];
q = [];

no = 1:(length(handles.lenses));


%iterate computation several times to allow for e.g. lens position 1 to
%depend on waist position 3.

for iter = 1:10
    
    oldwp = wp;
    oldlp = lp;
    olddw = dw;
    olddl = dl;
    oldno = no;

    %compute beam parameters in front of first lens


    for j = 1:length(handles.beams)

        beam = handles.beams(j);
        try
            leff(j) = evalin('base', beam.lambda);
        catch
            errordlg(sprintf('Error evaluating "%s" in lambda of beam %d:\n%s', beam.lambda, j, lasterr));
            return
        end
        try
            leff(j) = leff(j)*evalin('base', beam.m2);
        catch
            errordlg(sprintf('Error evaluating "%s" in M^2 of beam %d:\n%s', beam.m2, j, lasterr));
            return
        end
        try
            win = evalin('base', beam.diam)/2;
        catch
            errordlg(sprintf('Error evaluating "%s" in diam of beam %d:\n%s', beam.diam, j, lasterr));
            return
        end
        try
            Rinv = 1/evalin('base', beam.r);
        catch
            errordlg(sprintf('Error evaluating "%s" in R of beam %d:\n%s', beam.r, j, lasterr));
            return
        end
        try
            z(j) = evalin('base', beam.z);
        catch
            errordlg(sprintf('Error evaluating "%s" in z of beam %d:\n%s', beam.z, j, lasterr));
            return
        end

        q(j) = 1/(Rinv - sqrt(-1)*leff(j)/(pi*win^2));

        wp(1, j) = z(j) - Rinv*win^4*pi^2/(leff(j)^2 + Rinv^2*win^4*pi^2);
        dw(1, j) = 2*sqrt(leff(j)^2*win^2/(leff(j)^2 + Rinv^2*win^4*pi^2));

    end


    assignin('base', 'wp', wp);
    assignin('base', 'lp', lp);
    assignin('base', 'dw', dw);
    assignin('base', 'dl', dl);

    assignin('base', 'cwldleff', leff); %to give the beam diameter function access to them


    %compute beam parameters after each lens


    for ii = 1:length(handles.lenses)
        
        i = no(ii);
        lens = handles.lenses(i);
        try
            lp(i) = evalin('base', lens.z);
        catch
            errordlg(sprintf('Error evaluating "%s" in position of lens %d:\n%s', lens.z, i, lasterr));
            return
        end
        try
            f(i) = evalin('base', lens.f);
        catch
            errordlg(sprintf('Error evaluating "%s" in f of lens %d:\n%s', lens.f, i, lasterr));
            return
        end

        for j = 1:length(handles.beams)

            if (handles.lensbeams(i, j))
                M = [1, 0; -1/f(i), 1] * [1, lp(i)-z(j); 0, 1];
            else
                M = [1, lp(i)-z(j); 0, 1];
            end
            q(j) = (M(1,1)*q(j) + M(1,2))/(M(2,1)*q(j) + M(2,2));
            z(j) = lp(i);

            Rinv = real(1/q(j));
            win = sqrt(-leff(j)/pi/imag(1/q(j)));
            wp(i+1, j) = z(j) - Rinv*win^4*pi^2/(leff(j)^2 + Rinv^2*win^4*pi^2);
            dw(i+1, j) = 2*sqrt(leff(j)^2*win^2/(leff(j)^2 + Rinv^2*win^4*pi^2));
            dl(i, j) = 2*win;

        end

        assignin('base', 'wp', wp);
        assignin('base', 'lp', lp);
        assignin('base', 'dw', dw);
        assignin('base', 'dl', dl);

    end
    
    [dummy, no] = sort(lp);
    
    if (all(no == oldno) && all(all(wp == oldwp)) && all(lp == oldlp) && all(all(dw == olddw)) && all(all(dl == olddl)))
        if (iter > 3)
            fprintf('Converged after %d iterations.\n', iter-1);
        end
        break;
    end

end

if (iter == 10)
    warndlg('Warning: results keep changing after 10 iterations. Check your expressions.');
end


%plot


try
    zmin = evalin('base', get(handles.edit_zmin, 'String'));
catch
    errordlg(sprintf('Error evaluating "%s" in z min:\n%s', get(handles.edit_zmin, 'String'), lasterr));
    return
end
try
    zmax = evalin('base', get(handles.edit_zmax, 'String'));
catch
    errordlg(sprintf('Error evaluating "%s" in z max:\n%s', get(handles.edit_zmax, 'String'), lasterr));
    return
end
try
    maxdmax = evalin('base', get(handles.edit_dmax, 'String'));
catch
    errordlg(sprintf('Error evaluating "%s" in d max:\n%s', get(handles.edit_dmax, 'String'), lasterr));
    return
end
dlr = dl;
dlr(find(handles.lensbeams == 0)) = NaN;
zs = linspace(zmin, zmax, 2000);
dmax = 1.2*max(max(dlr));

figure(evalin('base', get(handles.edit_figure, 'String')));
if (get(handles.checkbox_keep, 'Value'))
    hold on;
else
    clf
    hold off;
end
plot([zmin, zmax], [0, 0], 'k-');
hold on;
colors = get(gca, 'ColorOrder');
for j = 1:length(handles.beams)
    plot(wp(:,j), 1000*dw(:,j), '^', 'Color', colors(mod(j-1,length(colors))+1,:));
    plot(lp, 1000*dlr(:,j), 'v', 'Color', colors(mod(j-1,length(colors))+1,:));
    ds = diam(zs, j);
    h(j) = plot(zs, 1000*ds, '-', 'Color', colors(mod(j-1,length(colors))+1,:));
    set(h(j), 'DisplayName', handles.beams(j).name);
    dmax = min(max(dmax, 1.2*max(ds)), maxdmax);
end
legend(h);
axis([zmin, zmax, -160*dmax, 1000*dmax]);
for i = 1:length(handles.lenses)
    text(lp(i), 1000*1.01*dmax, sprintf('%d\n%5.3f m\nf = %0.0f mm', i, lp(i), 1000*f(i)), 'HorizontalAlignment', 'Center', 'VerticalAlignment', 'Bottom'); 
    for j = 1:length(handles.beams)
        if (handles.lensbeams(i, j))
            text(lp(i), 1000*max(dlr(i,:)) + j*35*dmax, sprintf('%5.3f mm', 1000*dl(i,j)), 'Color', 0.7*colors(mod(j-1,length(colors))+1,:), 'HorizontalAlignment', 'Center');
        end
    end
end
for i = 1:size(wp, 1)
    for j = 1:length(handles.beams)
        if (i == 1 || handles.lensbeams(i-1, j))
            text(wp(i,j), 1000*min(dw(i,:)) - (j*105 - 35)*dmax, sprintf('%5.3f m\n+- %3.1f mm\n%5.3f mm', wp(i,j), 1000*pi*(dw(i,j)/2)^2/leff(j), 1000*dw(i,j)), 'Color', 0.7*colors(mod(j-1,length(colors))+1,:), 'HorizontalAlignment', 'Center');
        end
    end
end
xlabel('z [m]');
ylabel('diameter [mm]');
grid on;
set(gca, 'XMinorTick', 'on', 'YMinorTick', 'on');



%collect data

try
    data1 = evalin('base', 'data1');
catch
    data1 = zeros(2, 0);
end
try
    data2 = evalin('base', 'data2');
catch
    data2 = zeros(2, 0);
end
dx1str = '';
dy1str = '';
dx2str = '';
dy2str = '';

if (get(handles.checkbox_collect1, 'Value'))
    dx1str = get(handles.edit_collect1_horizontal, 'String');
    try
        dx = evalin('base', dx1str);
    catch
        errordlg(sprintf('Error evaluating "%s" in horizontal 1:\n%s', dx1str, lasterr));
        return
    end
    dy1str = get(handles.edit_collect1_vertical, 'String');
    try
        dy = evalin('base', dy1str);
    catch
        errordlg(sprintf('Error evaluating "%s" in vertical 1:\n%s', dy1str, lasterr));
        return
    end
    data1 = sortrows([data1, [dx; dy]]', 1)';
end

if (get(handles.checkbox_collect2, 'Value'))
    dx2str = get(handles.edit_collect2_horizontal, 'String');
    try
        dx = evalin('base', dx2str);
    catch
        errordlg(sprintf('Error evaluating "%s" in horizontal 2:\n%s', dx2str, lasterr));
        return
    end
    dy2str = get(handles.edit_collect2_vertical, 'String');
    try
        dy = evalin('base', dy2str);
    catch
        errordlg(sprintf('Error evaluating "%s" in vertical 2:\n%s', dy2str, lasterr));
        return
    end
    data2 = sortrows([data2, [dx; dy]]', 1)';
end

assignin('base', 'data1', data1);
assignin('base', 'data2', data2);

if (get(handles.checkbox_collect_separate, 'Value'))
    if (size(data1, 2) > 0)
        figure(evalin('base', get(handles.edit_collect_figure, 'String')));
        subplot(2, 1, 1);
        plot(data1(1,:), data1(2,:), 'o-');
        xlabel(dx1str);
        ylabel(dy1str);
        grid on;
        set(gca, 'XMinorTick', 'on', 'YMinorTick', 'on');
    end
    if (size(data2, 2) > 0)
        figure(evalin('base', get(handles.edit_collect_figure, 'String')));
        subplot(2, 1, 2);
        plot(data2(1,:), data2(2,:), 'o-');
        xlabel(dx2str);
        ylabel(dy2str);
        grid on;
        set(gca, 'XMinorTick', 'on', 'YMinorTick', 'on');
    end
else
    if (size(data1, 2) + size(data2, 2) > 0)
        figure(evalin('base', get(handles.edit_collect_figure, 'String')));
        subplot(1, 1, 1);
        plot(data1(1,:), data1(2,:), 'o-', data2(1,:), data2(2,:), 'o-');
        if (strcmp(dx2str, '') || strcmp(dx1str, dx2str))
            xlabel(dx1str);
        elseif (strcmp(dx1str, ''))
            xlabel(dx2str);
        else
            xlabel([dx1str, '; ', dx2str]);
        end
        if (strcmp(dy1str, ''))
            ylabel(dy2str);
        elseif (strcmp(dy2str, ''))
            ylabel(dy1str);
        else
            legend(dy1str, dy2str);
        end
        grid on;
        set(gca, 'XMinorTick', 'on', 'YMinorTick', 'on');
    end
end
